@@ -18,12 +18,33 @@ from six import PY2
 from six.moves import filter, filterfalse, map, range, zip, zip_longest
 
 __all__ = [
-    'accumulate', 'all_equal', 'consume', 'dotproduct', 'first_true',
-    'flatten', 'grouper', 'iter_except', 'ncycles', 'nth', 'padnone',
-    'pairwise', 'partition', 'powerset', 'quantify',
-    'random_combination_with_replacement', 'random_combination',
-    'random_permutation', 'random_product', 'repeatfunc', 'roundrobin',
-    'tabulate', 'tail', 'take', 'unique_everseen', 'unique_justseen'
+    'accumulate',
+    'all_equal',
+    'consume',
+    'dotproduct',
+    'first_true',
+    'flatten',
+    'grouper',
+    'iter_except',
+    'ncycles',
+    'nth',
+    'nth_combination',
+    'padnone',
+    'pairwise',
+    'partition',
+    'powerset',
+    'quantify',
+    'random_combination_with_replacement',
+    'random_combination',
+    'random_permutation',
+    'random_product',
+    'repeatfunc',
+    'roundrobin',
+    'tabulate',
+    'tail',
+    'take',
+    'unique_everseen',
+    'unique_justseen',
 ]
 
 
@@ -91,8 +112,7 @@ def tabulate(function, start=0):
 
 
 def tail(n, iterable):
-    """
-    Return an iterator over the last *n* items of *iterable*.
+    """Return an iterator over the last *n* items of *iterable*.
 
         >>> t = tail(3, 'ABCDEFG')
         >>> list(t)
@@ -281,7 +301,9 @@ def roundrobin(*iterables):
         >>> list(roundrobin('ABC', 'D', 'EF'))
         ['A', 'D', 'E', 'B', 'F', 'C']
 
-    See :func:`interleave_longest` for a slightly faster implementation.
+    This function produces the same output as :func:`interleave_longest`, but
+    may perform better for some inputs (in particular when the number of
+    iterables is small).
 
     """
     # Recipe credited to George Sakkis
@@ -303,7 +325,7 @@ def partition(pred, iterable):
     """
     Returns a 2-tuple of iterables derived from the input iterable.
     The first yields the items that have ``pred(item) == False``.
-    The first yields the items that have ``pred(item) == False``.
+    The second yields the items that have ``pred(item) == True``.
 
         >>> is_odd = lambda x: x % 2 != 0
         >>> iterable = range(10)
@@ -325,7 +347,7 @@ def powerset(iterable):
 
     """
     s = list(iterable)
-    return chain.from_iterable(combinations(s, r) for r in range(len(s)+1))
+    return chain.from_iterable(combinations(s, r) for r in range(len(s) + 1))
 
 
 def unique_everseen(iterable, key=None):
@@ -351,7 +373,7 @@ def unique_everseen(iterable, key=None):
                 if element not in seenset:
                     seenset_add(element)
                     yield element
-            except TypeError as e:
+            except TypeError:
                 if element not in seenlist:
                     seenlist_add(element)
                     yield element
@@ -362,7 +384,7 @@ def unique_everseen(iterable, key=None):
                 if k not in seenset:
                     seenset_add(k)
                     yield element
-            except TypeError as e:
+            except TypeError:
                 if k not in seenlist:
                     seenlist_add(k)
                     yield element
@@ -490,3 +512,39 @@ def random_combination_with_replacement(iterable, r):
     n = len(pool)
     indices = sorted(randrange(n) for i in range(r))
     return tuple(pool[i] for i in indices)
+
+
+def nth_combination(iterable, r, index):
+    """Equivalent to ``list(combinations(iterable, r))[index]``.
+
+    The subsequences of *iterable* that are of length *r* can be ordered
+    lexicographically. :func:`nth_combination` computes the subsequence at
+    sort position *index* directly, without computing the previous
+    subsequences.
+
+    """
+    pool = tuple(iterable)
+    n = len(pool)
+    if (r < 0) or (r > n):
+        raise ValueError
+
+    c = 1
+    k = min(r, n - r)
+    for i in range(1, k + 1):
+        c = c * (n - k + i) // i
+
+    if index < 0:
+        index += c
+
+    if (index < 0) or (index >= c):
+        raise IndexError
+
+    result = []
+    while r:
+        c, n, r = c * r // n, n - 1, r - 1
+        while index >= c:
+            index -= c
+            c, n = c * (n - r) // n, n - 1
+        result.append(pool[-1 - n])
+
+    return tuple(result)
